package com.isu.gaswellwatch.utils;

import cn.hutool.core.lang.Dict;
import cn.hutool.extra.expression.ExpressionUtil;
import com.isu.gaswellwatch.exception.BusinessException;
import com.isu.gaswellwatch.vo.UnitTreeVO;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


public class UnitsUtils {

    /**
     * 四舍五入
     *
     * @param value    - 原始数值
     * @param original - 原始单位
     * @param need     - 转换的单位
     * @return BigDecimal - 转换后的值
     */
    public static BigDecimal conversion(BigDecimal value, String original, String need) {
        return conversion(value, original, need, 2);
    }

    /**
     * 指定小数点位数，四舍五入
     *
     * @param value    - 原始数值
     * @param original - 原始单位
     * @param need     - 转换的单位
     * @param scale    - 小数点位数
     * @return BigDecimal - 转换后的值
     */
    public static BigDecimal conversion(BigDecimal value, String original, String need, final Integer scale) {
        //适应一个单位在多个分类中的情况，但不支持一个分类中有相同两个单位的情况
        List<UnitsEnum> unitEnums = getUnitEnum(original);
        UnitsEnum originalUnitEnum = null;
        UnitsEnum needUnitEnum = null;
        for (UnitsEnum unitsEnum : unitEnums) {
            UnitsEnum tempUnitEnum = getUnitEnum(need, unitsEnum);
            if (tempUnitEnum != null && tempUnitEnum != UnitsEnum.UN_KNOWN) {
                needUnitEnum = tempUnitEnum;
                originalUnitEnum = unitsEnum;
                break;
            }
        }
        if (needUnitEnum == null) throw new BusinessException(original + "不支持转计量单位:" + need);
        return conversion(value, originalUnitEnum, needUnitEnum, scale);
    }

    /**
     * 使用默认为数（2 或 need 的小数点位数）四舍五入
     *
     * @param value    - 原始数值
     * @param original - 原始单位
     * @param need     - 转换的单位
     * @return BigDecimal - 转换后的值
     */
    public static BigDecimal conversion(BigDecimal value, UnitsEnum original, UnitsEnum need) {
        String needString = need.conversionExpression;
        int scale = 2;
        if (needString.indexOf('.') > 0) {
            scale = needString.substring(needString.indexOf('.')).length() - 1;
        }
        return conversion(value, original, need, scale);
    }

    /**
     * 转换主方法，指定小数点位数，四舍五入
     *
     * @param value    - 原始数值
     * @param original - 原始单位
     * @param need     - 转换的单位
     * @param scale    - 小数点位数
     * @return BigDecimal - 数值
     */
    public static BigDecimal conversion(BigDecimal value, UnitsEnum original, UnitsEnum need, final Integer scale) {
        if (original == UnitsEnum.UN_KNOWN || need == UnitsEnum.UN_KNOWN) {
            throw new BusinessException("存在不支持的计量单位参数");
        }
        if (original.category != need.category) {
            throw new BusinessException(String.format("转换计量单位不统一 %s:%s 不能转换为 %s:%s ", original.category.name, original.units, need.category.name, need.units));
        }
        if (value == null) {
            return BigDecimal.ZERO;
        }
        //转单位基数值
        String originalExpression = original.getConversionBaseUnitExpression();
        String conversionExpression = need.getConversionBaseUnitExpression();
        if (StringUtils.isBlank(originalExpression) || StringUtils.isBlank(conversionExpression))
            return BigDecimal.ZERO;
        if (!originalExpression.contains("a") && !conversionExpression.contains("a")) {
            //都是倍数转换
            return value.multiply(new BigDecimal(conversionExpression)).divide(new BigDecimal(originalExpression), scale, RoundingMode.HALF_UP);
        }
        //非倍数转换,表达式计算
        conversionExpression = need.getConversionExpression();
        //转基数的情况
        if (StringUtils.isBlank(conversionExpression)) conversionExpression = need.getConversionBaseUnitExpression();
        value = evaluate(originalExpression, value);
        return evaluate(conversionExpression, value).setScale(scale, RoundingMode.HALF_UP);
    }

    private static BigDecimal evaluate(String expression, BigDecimal value) {
        try {
            if (!expression.contains("a")) {
                return value.divide(new BigDecimal(expression));
            } else {
                Dict dict = Dict.create()
                        .set("a", value);
                return new BigDecimal(ExpressionUtil.eval(expression, dict).toString());
            }
        } catch (Exception e) {
            throw new BusinessException("表达式计算异常");
        }

    }

    public static void main(String[] args) {
        long s = System.currentTimeMillis();
        System.out.println(conversion(new BigDecimal(1), "kgCO2e/MJ", "tCO2e/MJ",4));
        System.out.println(System.currentTimeMillis() - s);

    }


    /**
     * 获取计量单位枚举
     *
     * @param unit           - 计量单位名称
     * @param unitsEnumParam 指定计量单位的类型，可以为空
     * @return UnitsEnum - 计量单位
     */
    public static UnitsEnum getUnitEnum(String unit, UnitsEnum unitsEnumParam) {
        if (StringUtils.isBlank(unit)) {
            throw new BusinessException("存在不支持的计量单位参数:" + unit);
        }
        if (unitsEnumParam != null) {
            for (UnitsEnum unitEnum : UnitsEnum.values()) {
                if (unitEnum.getCategory() == unitsEnumParam.getCategory() && unitEnum.getUnits().equals(unit)) {
                    return unitEnum;
                }
            }
        } else {
            for (UnitsEnum unitEnum : UnitsEnum.values()) {
                if (unitEnum.getUnits().equals(unit)) {
                    return unitEnum;
                }
            }
        }
        return UnitsEnum.UN_KNOWN;
    }

    /**
     * 获取计量单位枚举
     *
     * @param unit - 计量单位名称
     * @return UnitsEnum - 计量单位
     */
    public static List<UnitsEnum> getUnitEnum(String unit) {
        if (StringUtils.isBlank(unit)) {
            throw new BusinessException("存在不支持的计量单位参数:" + unit);
        }
        List<UnitsEnum> unitsEnums = new ArrayList<>();
        for (UnitsEnum unitEnum : UnitsEnum.values()) {
            if (unitEnum.getUnits().equals(unit)) {
                unitsEnums.add(unitEnum);
            }
        }
        if (unitsEnums.size() == 0) throw new BusinessException("存在不支持的计量单位参数:" + unit);
        return unitsEnums;
    }

    @Getter
    public enum UnitsEnum {
        /**
         * 长度计量单位
         */
        LG_KM(CategoryEnum.LENGTH, "km", "0.001", "千米"),
        LG_M(CategoryEnum.LENGTH, "m", "1", "米"),
        LG_DM(CategoryEnum.LENGTH, "dm", "10", "分米"),
        LG_CM(CategoryEnum.LENGTH, "cm", "100", "厘米"),
        LG_MM(CategoryEnum.LENGTH, "mm", "1000", "毫米"),
        LG_UM(CategoryEnum.LENGTH, "um", "1000000", "微米"),
        LG_NM(CategoryEnum.LENGTH, "nm", "1000000000", "纳米"),
        LG_INCH(CategoryEnum.LENGTH, "in", "39.3700787", "英寸"),
        LG_FOOT(CategoryEnum.LENGTH, "ft", "3.2808399", "英尺"),
        LG_MILES(CategoryEnum.LENGTH, "mi", "0.00062137", "英里"),
        LG_NAUTICAL_MILE(CategoryEnum.LENGTH, "nmile", "0.00053996", "海里"),

        /**
         * 温度计量单位
         */
        TG_DEGREE_CELSIUS(CategoryEnum.TEMPERATURE, "℃", "1", "摄氏度"),
        TG_FAHRENHEIT_SCALE(CategoryEnum.TEMPERATURE, "℉", "(a-32)*5/9", "(a*9/5) + 32", "华氏度"),
        TG_KELVIN_SCALE(CategoryEnum.TEMPERATURE, "K", "a-273.15", "a+273.15", "开氏度"),

        /**
         * 质量计量单位
         */
        EG_T(CategoryEnum.WEIGHT, "t", "0.001", "吨"),
        EG_KG(CategoryEnum.WEIGHT, "kg", "1", "千克"),
        EG_G(CategoryEnum.WEIGHT, "g", "1000", "克"),
        EG_MG(CategoryEnum.WEIGHT, "mg", "1000000", "毫克"),
        EG_UG(CategoryEnum.WEIGHT, "μg", "1000000000", "微克"),
        EG_LB(CategoryEnum.WEIGHT, "lb", "2.2046226", "磅"),
        EG_OZ(CategoryEnum.WEIGHT, "oz", "35.2739619", "盎司"),
        EG_CT(CategoryEnum.WEIGHT, "ct", "5000", "克拉"),


        /**
         * 速度计量单位
         */
        TS_KM(CategoryEnum.SPEED, "km/h", "1", "公里/小时"),
        TS_M(CategoryEnum.SPEED, "m/s", "3.6", "米/秒"),
        TS_MILES(CategoryEnum.SPEED, "mi/h", "0.62137119", "英里/小时"),
        TS_NAUTICAL_MILE(CategoryEnum.SPEED, "nmile/h", "0.5399568", "海里/小时"),
        TS_KNOT(CategoryEnum.SPEED, "knot", "0.5399568", "节"),
        TS_MACH(CategoryEnum.SPEED, "mach", "0.00080985", "马赫"),


        /**
         * 面积计量单位
         */
        AR_M2(CategoryEnum.AREA, "m²", "1", "平方米"),
        AR_KM2(CategoryEnum.AREA, "km²", "0.000001", "平方千米"),

        /**
         * 体积计量单位
         */
        VOL_M3(CategoryEnum.VOLUME, "m³", "1", "立方米"),
        VOL_T(CategoryEnum.VOLUME, "t", "1", "吨"),
        VOL_DM3(CategoryEnum.VOLUME, "dm³", "1000", "立方分米"),
        VOL_CM3(CategoryEnum.VOLUME, "cm³", "1000000", "立方厘米"),
        VOL_MM3(CategoryEnum.VOLUME, "mm³", "1000000000", "立方毫米"),
        VOL_L(CategoryEnum.VOLUME, "l", "1000", "升"),
        VOL_DL(CategoryEnum.VOLUME, "dl", "10000", "分升"),
        VOL_CL(CategoryEnum.VOLUME, "cl", "100000", "厘升"),
        VOL_ML(CategoryEnum.VOLUME, "ml", "1000000", "毫升"),

        /**
         * 压强计量单位
         */
        P_ATM(CategoryEnum.PRESSURE, "atm", "1", "标准大气压"),
        P_PA(CategoryEnum.PRESSURE, "Pa", "101325", "帕斯卡"),
        P_HPA(CategoryEnum.PRESSURE, "hPa", "1013.25", "百帕"),
        P_KPA(CategoryEnum.PRESSURE, "KPa", "101.325", "千帕"),
        P_MPA(CategoryEnum.PRESSURE, "MPa", "0.101325", "兆帕"),

        /**
         * 有功功率计量单位
         */
        AP_W(CategoryEnum.ACTIVE_POWER, "W", "1000", "瓦"),
        AP_KW(CategoryEnum.ACTIVE_POWER, "kW", "1", "千瓦"),
        AP_MW(CategoryEnum.ACTIVE_POWER, "MW", "0.001", "兆瓦"),

        /**
         * 无功功率计量单位
         */
        RP_VAR(CategoryEnum.REACTIVE_POWER, "Var", "1000", "乏尔"),
        RP_KVAR(CategoryEnum.REACTIVE_POWER, "kVar", "1", "千乏尔"),
        RP_MVAR(CategoryEnum.REACTIVE_POWER, "MVar", "0.001", "兆乏尔"),

        /**
         * 视在功率计量单位
         */
        VA_VA(CategoryEnum.APPARENT_POWER, "VA", "1000", "伏安"),
        VA_KVA(CategoryEnum.APPARENT_POWER, "kVA", "1", "千伏安"),
        VA_MVA(CategoryEnum.APPARENT_POWER, "MVA", "0.001", "兆伏安"),


        /**
         * 时间
         */
        T_YEAR(CategoryEnum.TIME, "yr", "1", "年"),
        T_MONTH(CategoryEnum.TIME, "month", "12", "月"),
        T_D(CategoryEnum.TIME, "d", "365", "日"),
        T_WEEK(CategoryEnum.TIME, "week", "52.1428571", "周"),
        T_HOUR(CategoryEnum.TIME, "h", "8760", "小时"),
        T_MINUTE(CategoryEnum.TIME, "min", "525600", "分钟"),
        T_SECOND(CategoryEnum.TIME, "s", "31536000", "秒"),

        /**
         * 角度
         */
        DEG_D(CategoryEnum.APPARENT_POWER, "°", "1", "度"),
        DEG_F(CategoryEnum.APPARENT_POWER, "'", "60", "分"),
        DEG_M(CategoryEnum.APPARENT_POWER, "\"", "3600", "秒"),

        /**
         * 密度
         */
        DY_MGM3(CategoryEnum.DENSITY, "mg/m³", "1", "毫克每立方米"),
        DY_UGM3(CategoryEnum.DENSITY, "μg/m³", "0.001", "微克每立方米"),
        DY_PPM(CategoryEnum.DENSITY, "ppm", "0.000001", "百万分比浓度"),
        DY_MGL(CategoryEnum.DENSITY, "mg/L", "1000", "毫克每升"),

        /**
         * 角度
         */
        TO_JOULE(CategoryEnum.THERMO, "J", "1", "焦耳"),
        TO_KILOJOULE(CategoryEnum.THERMO, "kJ", "0.001", "千焦"),
        TO_MEGAJOULE(CategoryEnum.THERMO, "MJ", "0.000001", "兆焦"),
        TO_GIGAJOULE(CategoryEnum.THERMO, "GJ", "0.000000001", "吉焦"),

        /**
         * 流量
         */
        F1_M3S(CategoryEnum.FLOW_ONE, "m³/s", "1", "立方米每秒"),
        F1_M3M(CategoryEnum.FLOW_ONE, "m³/min", "0.06", "立方米每分钟"),
        F1_M3H(CategoryEnum.FLOW_ONE, "m³/h", "0.001", "立方米每小时"),


        F2_LS(CategoryEnum.FLOW_TWO, "L/s", "1", "升每秒"),
        F2_LM(CategoryEnum.FLOW_TWO, "L/min", "60", "升每分钟"),
        F2_LH(CategoryEnum.FLOW_TWO, "L/h", "3600", "升每小时"),


        /**
         * 照度
         */
        ILL_LM(CategoryEnum.ILLUMINANCE, "1Lm/m²", "1", "平方米流明"),
        ILL_LX(CategoryEnum.ILLUMINANCE, "Lx", "1", "勒克斯"),

        /**
         * 电流计量单位
         */
        CURRENT_A(CategoryEnum.CURRENT, "A", "1", "安"),
        CURRENT_MA(CategoryEnum.CURRENT, "mA", "0.001", "毫安"),
        CURRENT_UA(CategoryEnum.CURRENT, "μA", "0.000001", "微安"),

        /**
         * 电压计量单位
         */
        VOLT_V(CategoryEnum.VOLTAGE, "V", "1", "伏特"),
        VOLT_KV(CategoryEnum.VOLTAGE, "Kv", "0.001", "千伏"),

        /**
         * 电量计量单位
         */
        POWER_W(CategoryEnum.POWER, "Wh", "1", "瓦时"),
        POWER_KW(CategoryEnum.POWER, "kWh", "0.001", "千瓦时"),
        POWER_MW(CategoryEnum.POWER, "MWh", "0.000001", "兆瓦时"),
        POWER_GW(CategoryEnum.POWER, "GWh", "0.000000001", "吉瓦时"),

        /**
         * 热当量
         */
        HC1_TCE(CategoryEnum.HEAT_CAPACITY1, "tce", "1", "吨标准煤"),
        HC1_KGCE(CategoryEnum.HEAT_CAPACITY1, "kgce", "1000", "千克标准煤"),

        HC2_TCE(CategoryEnum.HEAT_CAPACITY2, "tce/m³", "1", "吨标准煤/立方米"),
        HC2_KGCE(CategoryEnum.HEAT_CAPACITY2, "kgce/m³", "1000", "千克标准煤/立方米"),

        HC3_TCE(CategoryEnum.HEAT_CAPACITY3, "tce/kg", "1", "吨标准煤/千克"),
        HC3_KGCE(CategoryEnum.HEAT_CAPACITY3, "kgce/kg", "1000", "千克标准煤/千克"),

        HC4_TCE(CategoryEnum.HEAT_CAPACITY4, "tce/MJ", "1", "吨标准煤/兆焦"),
        HC4_KGCE(CategoryEnum.HEAT_CAPACITY4, "kgce/MJ", "1000", "千克标准煤/兆焦"),

        HC5_TCE(CategoryEnum.HEAT_CAPACITY5, "tce/GJ", "1", "吨标准煤/吉焦"),
        HC5_KGCE(CategoryEnum.HEAT_CAPACITY5, "kgce/GJ", "1000", "千克标准煤/吉焦"),

        HC6_TCE(CategoryEnum.HEAT_CAPACITY6, "tce/t", "1", "吨标准煤/吨"),
        HC6_KGCE(CategoryEnum.HEAT_CAPACITY6, "kgce/t", "1000", "千克标准煤/吨"),

        HC7_TCE(CategoryEnum.HEAT_CAPACITY7, "tce/L", "1", "吨标准煤/升"),
        HC7_KGCE(CategoryEnum.HEAT_CAPACITY7, "kgce/L", "1000", "千克标准煤/升"),

        HC8_TCE(CategoryEnum.HEAT_CAPACITY8, "tce/kWh", "1", "吨标准煤/千瓦时"),
        HC8_KGCE(CategoryEnum.HEAT_CAPACITY8, "kgce/kWh", "1000", "千克标准煤/千瓦时"),

        CEF1_TCO(CategoryEnum.CEF1, "tCO2e", "1", "吨二氧化碳当量"),
        CEF1_KGCO(CategoryEnum.CEF1, "KgCO2e", "1000", "千克二氧化碳当量"),

        CEF2_TCO(CategoryEnum.CEF2, "tCO2e/m³", "1", "吨二氧化碳当量/立方米"),
        CEF2_KGCO(CategoryEnum.CEF2, "KgCO2e/m³", "1000", "千克二氧化碳当量/立方米"),

        CEF3_TCO(CategoryEnum.CEF3, "tCO2e/t", "1", "吨二氧化碳当量/吨"),
        CEF3_KGCO(CategoryEnum.CEF3, "KgCO2e/t", "1000", "千克二氧化碳当量/吨"),

        CEF4_TCO(CategoryEnum.CEF4, "tCO2e/kg", "1", "吨二氧化碳当量/千克"),
        CEF4_KGCO(CategoryEnum.CEF4, "KgCO2e/kg", "1000", "千克二氧化碳当量/千克"),

        CEF5_TCO(CategoryEnum.CEF5, "tCO2e/L", "1", "吨二氧化碳当量/升"),
        CEF5_KGCO(CategoryEnum.CEF5, "KgCO2e/L", "1000", "千克二氧化碳当量/升"),

        CEF6_TCO(CategoryEnum.CEF6, "tCO2e/MWh", "1", "吨二氧化碳当量/千瓦时"),
        CEF6_KGCO(CategoryEnum.CEF6, "KgCO2e/kWh", "1000", "千克二氧化碳当量/千瓦时"),

        CEF7_TCO(CategoryEnum.CEF7, "tCO2e/GJ", "1", "吨二氧化碳当量/吉焦"),
        CEF7_KGCO(CategoryEnum.CEF7, "KgCO2e/GJ", "1000", "千克二氧化碳当量/吉焦"),
        CEF7_TCOMJ(CategoryEnum.CEF7, "tCO2e/MJ", "0.001", "吨二氧化碳每兆焦"),
        CEF7_KGCOMJ(CategoryEnum.CEF7, "kgCO2e/MJ", "1", "千克二氧化碳每兆焦"),
        /**
         * 未知
         */
        UN_KNOWN(CategoryEnum.UN_KNOWN, "未知", "0", "未知");

        /**
         * 计量类别
         */
        private final CategoryEnum category;
        /**
         * 计量单位
         */
        private final String units;

        /**
         * 换算表达式
         */
        private final String conversionBaseUnitExpression;

        /**
         * 换算表达式
         */
        private final String conversionExpression;
        /**
         * 描述
         */
        private final String description;


        UnitsEnum(CategoryEnum category, String units, String conversionBaseUnitExpression, String description) {
            this.category = category;
            this.units = units;
            this.conversionBaseUnitExpression = conversionBaseUnitExpression;
            this.description = description;
            this.conversionExpression = ""; // 设置默认值为空字符串
        }

        UnitsEnum(CategoryEnum category, String units, String conversionBaseUnitExpression, String conversionExpression, String description) {
            this.category = category;
            this.units = units;
            this.conversionBaseUnitExpression = conversionBaseUnitExpression;
            this.conversionExpression = conversionExpression;
            this.description = description;
        }
    }

    /**
     * 计量类别
     */
    @Getter
    public enum CategoryEnum {
        /**
         * 长度
         */
        LENGTH("Length", "长度"),
        /**
         * 质量
         */
        WEIGHT("Weight", "质量"),
        /**
         * 温度
         */
        TEMPERATURE("Temperature", "温度"),
        /**
         * 速度
         */
        SPEED("Speed", "速度"),

        AREA("Area", "面积"),

        VOLUME("Volume", "体积"),

        PRESSURE("Pressure", "压强"),

        ACTIVE_POWER("ActivePower", "有功功率"),

        REACTIVE_POWER("ReactivePower", "无功功率"),

        APPARENT_POWER("ApparentPower", "视在功率"),

        TIME("Time", "时间"),

        ANGLE("Angle", "角度"),

        /**
         * 密度
         */
        DENSITY("Density", "密度"),
        /**
         * 热工
         */
        THERMO("Thermo", "热工"),
        /**
         * 流量
         */
        FLOW_ONE("Flow1", "流量(立方米)"),

        FLOW_TWO("Flow2", "流量(升)"),

        ILLUMINANCE("Illuminance", "照度"),
        /**
         * 电流
         */
        CURRENT("Current", "电流"),

        /**
         * 电压
         */
        VOLTAGE("Voltage", "电压"),

        /**
         * 电量
         */
        POWER("Power", "电量"),
        UN_KNOWN("un_known", "未知"),

        HEAT_CAPACITY1("HeatCapacity1", "热当量"),
        HEAT_CAPACITY2("HeatCapacity2", "热当量"),
        HEAT_CAPACITY3("HeatCapacity3", "热当量"),
        HEAT_CAPACITY4("HeatCapacity4", "热当量"),
        HEAT_CAPACITY5("HeatCapacity5", "热当量"),
        HEAT_CAPACITY6("HeatCapacity6", "热当量"),
        HEAT_CAPACITY7("HeatCapacity7", "热当量"),
        HEAT_CAPACITY8("HeatCapacity8", "热当量"),

        CEF1("CarbonEmissionFactor1", "碳排放因子"),
        CEF2("CarbonEmissionFactor2", "碳排放因子"),
        CEF3("CarbonEmissionFactor3", "碳排放因子"),
        CEF4("CarbonEmissionFactor4", "碳排放因子"),
        CEF5("CarbonEmissionFactor5", "碳排放因子"),
        CEF6("CarbonEmissionFactor6", "碳排放因子"),
        CEF7("CarbonEmissionFactor7", "碳排放因子"),
        CEF8("CarbonEmissionFactor8", "碳排放因子"),
        ;
        /**
         * 类别名称
         */
        private final String name;
        /**
         * 描述
         */
        private final String description;

        CategoryEnum(String name, String description) {
            this.name = name;
            this.description = description;
        }
    }


    public static List<String> getUnitList(String category) {
        UnitsEnum[] units = UnitsEnum.values();
        if (StringUtils.isNotBlank(category)) {
            List<String> categoryByDes = getCategoryByDes(category);
            if (CollectionUtils.isEmpty(categoryByDes)) return new ArrayList<>();
            return Arrays.stream(units).filter(unitsEnum ->
                            categoryByDes.contains(unitsEnum.getCategory().getName()))
                    .map(UnitsEnum::getUnits).collect(Collectors.toList());
        }
        return Arrays.stream(units).map(UnitsEnum::getUnits).collect(Collectors.toList());
    }

    public static List<String> getCategoryByDes(String description) {
        CategoryEnum[] values = CategoryEnum.values();
        return Arrays.stream(values).filter(categoryEnum -> categoryEnum.getDescription().equals(description)).map(CategoryEnum::getName).collect(Collectors.toList());
    }

    /**
     * 获取计量单位类型列表
     */
    public static List<String> getCategoryList() {
        CategoryEnum[] values = CategoryEnum.values();
        return Arrays.stream(values).filter(s-> !s.getDescription().equals("未知")).map(CategoryEnum::getDescription).distinct().collect(Collectors.toList());
    }
    public static List<UnitTreeVO> buildTree(){
        List<UnitTreeVO> result=new ArrayList<>();
        List<String> categoryList = getCategoryList();
        for (String category : categoryList) {
            result.add(UnitTreeVO.builder().label(category).children(getUnitList(category)).build());
        }
        return result;
    }

}
